import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class Plotter extends JPanel
{
   private int plotterLength = 1200, plotterHeight = 600;

   private JFrame frame = new JFrame();

   private IDatenquelle datenquelle;

   public Plotter() 
   {
      frame.setSize(plotterLength, plotterHeight);
      frame.setResizable(false);
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.add(this);
   }

   // Datenquelle wird ueber diese Methode 'injiziert'
   public void setDatenquelle(IDatenquelle datenquelle) 
   {
      this.datenquelle = datenquelle;
   }

   public void plot() 
   {
      // Wenn keine Datenquelle in der Registratur vorhanden ist,
      // beende den Plotvorgang
      if (datenquelle == null)
         return;
      
      frame.setVisible(true);

      // die fuer JFrame angegebene Groesse bezieht sich auf die vom
      // Betriebssystem zugewiesene Fenstergroesse.
      // Daher wird die tatsaechliche Plottergroesse erst nachtraeglich
      // gesetzt.
      Dimension tatsaechlicheGroesse = frame.getContentPane().getSize();
      plotterLength = (int) tatsaechlicheGroesse.getWidth();
      plotterHeight = (int) tatsaechlicheGroesse.getHeight();
   }

   @Override
   protected void paintComponent(Graphics g) 
   {
      super.paintComponent(g);
      ArrayList<Point> datenreihe = datenquelle.getDatenreihe();

      // Wenn weniger als zwei Punkte in Datenreihe, verlasse Funktion
      if (datenreihe == null || datenreihe.size() <= 1)
         return;

      // Minimum und Maximum-Punkte der x- und y-Reihe bestimmen und sichern
      Point minPunkte = getMinPunkte(datenreihe);
      Point maxPunkte = getMaxPunkte(datenreihe);

      // Falls es keine Steigung gibt, vergroessere den y-Bereich des Plotters
      if (minPunkte.y == maxPunkte.y) 
      {
         minPunkte.y -= 1;
         maxPunkte.y += 1;
      }

      // Berechne einen Offset um Ueberschneidungen der Axenbeschriftung
      // zu verhindern.
      int xOffset = (int) (plotterLength * 0.05) / 2;
      int yOffset = (int) (plotterHeight * 0.05) / 2;

      // Berechne Pixel pro Einheit, um die Position der Datenpunkte bestimmen
      // zu koennen
      Point pixelProEinheit = new Point((int) (plotterLength / (maxPunkte.x - minPunkte.x)), (int) (plotterHeight / (maxPunkte.y - minPunkte.y)));

      for (int i = 0; i < datenreihe.size(); i++) 
      {
         // Berechne Positionkoordinaten des aktuellen Punktes auf der Plotteroberflaeche 
         Point aktuellerPunkt = new Point((int) ((datenreihe.get(i).x - minPunkte.x) * pixelProEinheit.x * 0.95) + xOffset,
            (int) (plotterHeight - (datenreihe.get(i).y - minPunkte.y) * pixelProEinheit.y * 0.95) - yOffset);

         // Linie zwischen Punkt und nachfolgenden Punkt einzeichnen
         if (i < (datenreihe.size() - 1)) 
         {
            g.setColor(Color.RED);
            g.drawLine(aktuellerPunkt.x, aktuellerPunkt.y, (int) ((datenreihe.get(i + 1).x - minPunkte.x) * pixelProEinheit.x * 0.95) + xOffset,
               (int) (plotterHeight - (datenreihe.get(i + 1).y - minPunkte.y) * pixelProEinheit.y * 0.95) - yOffset);
         }

         // Schwarz als Beschriftungsfarbe auswaehlen
         g.setColor(Color.BLACK);

         // Beschriftung der y-Achsenpunkte einzeichnen
         g.drawLine(0, aktuellerPunkt.y, 3, aktuellerPunkt.y);
         g.drawString("" + datenreihe.get(i).y, 6, aktuellerPunkt.y - 2);

         // Beschriftung der x-Achsenpunkte einzeichnen
         g.drawLine(aktuellerPunkt.x, plotterHeight, aktuellerPunkt.x, plotterHeight - 4);
         g.drawString("" + datenreihe.get(i).x, aktuellerPunkt.x + 3, plotterHeight - 4);

         // Punkt markieren
         g.drawOval(aktuellerPunkt.x - 3, aktuellerPunkt.y - 3, 6, 6);
      }
   }

   Point getMaxPunkte(ArrayList<Point> datenreihe) 
   {
      Point maxPunkte = new Point(Integer.MIN_VALUE, Integer.MIN_VALUE);
      for (Point datenpunkt : datenreihe) 
      {
         if (datenpunkt.x > maxPunkte.x)
            maxPunkte.x = datenpunkt.x;
         if (datenpunkt.y > maxPunkte.y)
            maxPunkte.y = datenpunkt.y;
      }
      return maxPunkte;
   }

   Point getMinPunkte(ArrayList<Point> datenreihe) 
   {
      Point minPunkte = new Point(Integer.MAX_VALUE, Integer.MAX_VALUE);
      for (Point datenpunkt : datenreihe) 
      {
         if (datenpunkt.x < minPunkte.x)
            minPunkte.x = datenpunkt.x;
         if (datenpunkt.y < minPunkte.y)
            minPunkte.y = datenpunkt.y;
      }
      return minPunkte;
   }
}